#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from POFile import *
from POFileStatus import *
from Status import *

class File(Status):
    def __init__(self, module, filename):
        Status.__init__(self, module.getConfig())        
        self.module = module
        self.filename = filename
        self.fileParser = None
        self.fileStatus = None
        self.translated, self.untranslated = 0, 0
        self.fuzzy, self.obsolete = 0, 0
        self.errors = None

        self.fileExt = self.getConfig().getFileSuffix()
        self.templateExt = self.getConfig().getTemplateSuffix()
        
        self.file = os.path.join(module.getFileDir(),
                                 filename + "." + self.fileExt)
        self.template = os.path.join(module.getTemplateDir(),
                                     filename + "." + self.templateExt)
        
    def getModule(self):
        return self.module
    
    def getFilename(self):
        return self.filename

    def getFileUrl(self):
        return os.path.join(self.module.getFileUrl(),
                            self.filename + "." + self.fileExt)

    def getTemplateUrl(self):
        return os.path.join(self.module.getTemplateUrl(),
                            self.filename + "." + self.templateExt)
    
    def isNew(self):
        # If a file is new, it means only exists as template
        return (os.path.exists(self.template) and
                not (os.path.exists(self.file)))

    def isObsolete(self):
        # If a file is obsolete, it means it doesn't exist as template
        return (os.path.exists(self.file) and
                not (os.path.exists(self.template)))

    def isValid(self):
        return (os.path.exists(self.file) and
                os.path.exists(self.template))

    def hasErrors(self):
        return (self.fileParser is None or self.fileParser.hasErrors())

    def xmlify(self, text):
        sequences = [('&','&amp;'), ('<','&lt;'), ('>','&gt;')]
        for seq in sequences:
            (original, entity) = seq
            text = string.replace(text, original, entity)

        return text
    
    def toXML(self, level = 0):
        state = "VALID"
        subContent = self.textTag("name", self.getFilename(), level + 1,
                                  self.attribute("file", self.getFileUrl()) +
                                  self.attribute("template", self.getTemplateUrl()))
                                  
        if (self.isNew()):
            state = "NEW"
        elif (self.isObsolete()):
            state = "OBSOLETE"
        else:
            status = self.run()
            if status:
                # List all the values for status (fuzzies, untranslated,....)
                subContent = subContent + self.openTag("status", level + 1)
                if (self.translated is not None and self.translated > 0):
                    subContent = subContent + \
                                 self.textTag("translated",
                                              str(self.translated),level + 2)

                if (self.untranslated is not None and self.untranslated > 0):
                    subContent = subContent + \
                                 self.textTag("untranslated",
                                              str(self.untranslated),level + 2)
                    
                if (self.fuzzy is not None and self.fuzzy > 0):
                    subContent = subContent + \
                                 self.textTag("fuzzy",
                                              str(self.fuzzy),level + 2)
                    
                if (self.obsolete is not None and self.obsolete > 0):
                    subContent = subContent + \
                                 self.textTag("obsolete",
                                              str(self.obsolete),level + 2)

                subContent = subContent + self.closeTag("status", level + 1)

                # Now, if the file has syntactic / semantical errors, display
                # those errors, including the line where they occured
                if (self.hasErrors()):
                    subContent = subContent + self.openTag("errors",level + 1)

                    for line, message in self.errors:
                        message = self.xmlify(message)
                        subContent = subContent + \
                                     self.textTag("error", message, level + 2,
                                                  self.attribute("line",line))
                    
                    subContent = subContent + self.closeTag("errors",level + 1)
                    
        content = self.openTag("file", level, self.attribute("state", state))+\
                  subContent + \
                  self.closeTag("file", level)        
        return content
        
    def run(self):
        sys.stderr.write("FILE INFO:" + "\n")
        sys.stderr.write("Name: [%s]" % self.filename + "\n")
        sys.stderr.write("Real Name: [%s]" % self.file + "\n")
        sys.stderr.write("Template Name: [%s]" % self.template + "\n")
        if (self.isValid()):
            sys.stderr.write("Status: [Exists]" + "\n")
            self.fileParser = POFile(self.file)
            self.fileStatus = POFileStatus(self.config, self.file)

            # Execute both status and parsing to find for errors
	    parserStatus = self.fileParser.parse()
            if parserStatus:
                sys.stderr.write("No Errors" + "\n")
                self.fileParser.check()
                if (self.fileParser.hasErrors()):
                    sys.stderr.write("File With Errors" + "\n")
                    self.errors = self.fileParser.getErrors()
                    for l, e in self.errors:
                        sys.stderr.write('%5d: %s' % (l, e) + "\n")

            statsStatus = self.fileStatus.run()
            if statsStatus:
                self.translated = self.fileStatus.getTranslated()
                self.fuzzy = self.fileStatus.getFuzzy()
                self.untranslated = self.fileStatus.getUntranslated()
                self.obsolete = self.fileStatus.getObsolete()
                sys.stderr.write("File With No Errors" + "\n")
                sys.stderr.write("Message Status" + "\n")
                sys.stderr.write(("Translated: [%s] Fuzzy: [%s] " + \
                                  "Untranslated: [%s] Obsolete: [%s]") % \
                                 (self.translated, self.fuzzy,
                                  self.untranslated, self.obsolete) +"\n")

            return (parserStatus or statsStatus) 
            
        elif (self.isObsolete()):
            sys.stderr.write("Status: [Obsolete]" + "\n")
            return 1
        elif (self.isNew()):
            sys.stderr.write("Status: [New]" + "\n")
            return 1
        else:
            sys.stderr.write("Status: [Invalid]" + "\n")
            return 0
        
