# -*- coding: utf-8 -*-

# PO file class
#
# Pedro Morais <morais@kde.org>
# José Nuno Pires <jncp@netcabo.pt>
# (c) Copyright 2003, 2004
# Distributable under the terms of the GPL - see COPYING

import capitalization

class POFile:

    def __init__(self, filename):
        self.filename = filename
        self.fuzzy = None
        self.untranslated = None
        self.translated = None
        self.validateError = None
        self.data = []
        self.errors = []
        self.allowCount = None
        self.enableCheckAccelerator = 1
        self.enableCheckCapitalization = 0
        self.accelerator = '&'
        self.enableCheckEndPontuation = 1
        self.enableCheckLineLength = 0
        self.spellExtra = ''

    def getRatio(self):
        if self.translated == None: return None   
        total = self.translated
        if self.fuzzy: total = total + self.fuzzy
        if self.untranslated: total = total + self.untranslated
        return float(self.translated) / float(total)

    def isFullyTranslated(self):
        return self.translated and not(self.fuzzy) and not(self.untranslated)

    def validate(self):
        import os
        command = "msgfmt --statistics -o /dev/null %s 2>&1" % self.filename
        output = os.popen(command).read()
        self.translated = self.__vextract(output, " translated")
        if self.translated == None: self.validateError = output
        self.fuzzy = self.__vextract(output, " fuzzy")
        self.untranslated = self.__vextract(output, " untranslated")
        return self.validateError == None
    
    def __vextract(self, output, token):
        end = output.find(token)
        if end == -1: return None
        start = output.rfind(" ", 0, end) + 1
        return int(output[start:end])

    def read_lines(self):
        try:
            pofile = open(self.filename)
            lines = pofile.readlines()
            pofile.close()
            return lines
        except IOError:
            return None

    def parse(self):
        lines = self.read_lines()
        if lines == None: return 0        
        current = ""
        msgid = ""
        mode = 0
        self.data = []
        line = 0
        msgidline = 0
        message = 0
        for i in lines:
            line = line + 1
            l = i.strip()
            if len(l) == 0 or i[0] == '#': continue
            if i[:6] == "msgid ":
                if mode == 2:
                    self.data.append((msgidline, message, msgid, current))
                current = ""
                mode = 1                
                msgidline = line
                message = message + 1
                l = l[6:]
            if i[:7] == "msgstr ":
                if mode == 1: msgid = current
                current = ""
                mode = 2
                l = l[7:]
            if mode: current = current + l[1:-1]
        return 1

    def replace(self, number, text, output):
        prepare = self.prepare_replace(number)
        if prepare == None: return 0
        if output != None:
            self.execute_replace(prepare, text, output)
        return 1

    def prepare_replace(self, number):
        lines = self.read_lines()
        if lines == None: return None
        
        line = 0
        message = 0        
        headline = None
        tailline = None
        
        for i in lines:
            line = line + 1
            l = i.strip()
            if len(l) == 0 or i[0] == '#':
                if headline and not(tailline):
                    tailline = line
                    break
                continue
            if i[:6] == "msgid ":
                message = message + 1
            if i[:7] == "msgstr ":
                if message == number:
                    headline = line
        if not(headline) or not(tailline): return None
        return (headline, tailline, lines)
        
    def execute_replace(self, prepare, text, output):
        headline, tailline, lines = prepare
        for i in lines[:headline - 1]: output.write(i)
        output.write('msgstr "%s"\n' % text)
        for i in lines[tailline - 1:]: output.write(i)

    def get_header_int(self, line, key, current):
        if line.startswith("X-POFile-%s: " % key):
            try:
                return int(line[len("X-POFile-%s: " % key):])
            except:
                pass
        return current
            
    def parseHeader(self):
        if len(self.data) > 0:
            headerLines = self.data[0][3].split('\\n')
            for i in headerLines:
                self.allowCount = self.get_header_int(
                    i, "Allow", self.allowCount)
                self.enableCheckAccelerator = self.get_header_int(
                    i, "CheckAccelerator", self.enableCheckAccelerator)
                self.enableCheckEndPontuation = self.get_header_int(
                    i, "CheckEndPontuation", self.enableCheckEndPontuation)
                self.enableCheckLineLength = self.get_header_int(
                    i, "CheckLineLength", self.enableCheckLineLength)
                if i.startswith("X-Spell-Extra: "):
                    self.spellExtra = (self.spellExtra +
                                       i[len("X-Spell-Extra: "):] + " ")

    def toWordList(self, s):
        s = s.replace('\\n', ' ').replace('\\t', ' ').replace(',', ' ').replace('.', ' ').replace('?', ' ').replace('!', ' ').replace(':', ' ').replace(';', ' ').replace('(', ' ').replace(')', ' ').replace('/', ' ').replace('&&', ' ').replace('&', '').replace('</', ' ').replace('<', ' ').replace('>', ' ').replace('\\"', " ")
        l = s.split()
        if l == None or len(l) < 1: return l
        ret = {}
        for i in l: ret[i.lower()] = i.lower()
        return ret.values()

    def findEquiv(self, minLength, equiv):
        for l, m, i, s in self.data:
            if len(i) == 0 or len(s) == 0: continue
            si = self.toWordList(i)
            if len(si) == 0: continue
            ss = self.toWordList(s)
            if len(ss) == 0: continue
            
            for word in si:
                if self.trueStringLen(word) > minLength:
                    countMap = {}
                    if equiv.has_key(word): countMap = equiv[word]
                    for corr in ss:
                        if self.trueStringLen(corr) > minLength:
                            count = 0
                            if countMap.has_key(corr): count = countMap[corr]
                            countMap[corr] = count + 1
                    equiv[word] = countMap
        return equiv

    def consistency(self, map, strip):
        if self.parse() == 0: return 0
        for line, message, msgid, msgstr in self.data:
            if len(msgid) > 0:
                for i in strip:
                    msgstr = msgstr.replace(i, '')
                    msgid = msgid.replace(i, '')
                cmsgid = map.get(msgid)
                add = (self.filename, message)
                if cmsgid == None:
                    map[msgid] = { msgstr: [add,] }
                else:
                    cmsgstr = cmsgid.get(msgstr)
                    if cmsgstr == None:
                        cmsgid[msgstr] = [add,]
                    else:
                        cmsgstr.append(add)
        return 1

    def check(self):
        self.errors = []
        for l, m, i, s in self.data:
            if len(i) == 0 or len(s) == 0: continue
            if self.enableCheckAccelerator:
                self.checkAccelerator(i, s, l, self.accelerator)
            if self.enableCheckCapitalization:
                self.checkCapitalization(i, s, l)
            if self.enableCheckEndPontuation:
                self.checkEndPontuation(i, s, l)
            if len(s) > 1 and s[:2] == "_:":
                self.errors.append((l, '_: on msgstr'))
            if s[-1] == "<" and i[-1] != '<':
                self.errors.append((l, '< trailling msgstr'))
            if i.count('\\n') == 0 and s.count('\\n') > 0:
                self.errors.append((l, 'extra \\n'))
            if self.enableCheckLineLength:
                self.checkLineLength(s, l, self.enableCheckLineLength,
                                     single_lines = 1)

    def checkAccelerator(self, i, s, pos, acc):
        if i.count(acc) == 1 and i.count(acc + ' ') == 0 and s.count(acc) == 0:
            self.errors.append((pos, 'missing %s acelerator' % acc))
        if i.count(acc) == 1 and i.count(acc + ' ') == 0 and s.count(acc) > 1:
            self.errors.append((pos, 'extra %s acelerator' % acc))
        if i.count(acc) == 0 and s.count(acc) == 1:
            self.errors.append((pos, 'extra %s acelerator' % acc))

    def checkCapitalization(self, i, s, pos):
        if i.startswith("_: NAME OF TRANSLATORS"): return
        if i.startswith("_: EMAIL OF TRANSLATORS"): return
        ic = capitalization.capitalization(i, (), ('to',))
        if ic != capitalization.CAP_UNKNOWN:
            sc = capitalization.capitalization(s, (), ('de', 'do',
                                                       'da', 'dos',
                                                       'das', 'o', 'a',
                                                       'e', 'os', 'as',
                                                       'pelo', 'para'))
            if sc != ic:
                self.errors.append((pos, 'capitalization error'))

    def checkEndPontuation(self, i, s, pos):
        if i[-1] == ':' and s[-1] != ':':
            self.errors.append((pos, 'missing :'))
        if len(i) > 1 and len(s) > 1 and i[-2:] == ": " and s[-2:] != ": ":
            self.errors.append((pos, 'missing : '))
        if len(i) > 2 and len(s) > 2 and i[-3:] == "..." and s[-3:] != "...":
            self.errors.append((pos, 'missing ...'))
        if i[-1] == '.' and s[-1] != '.':
            self.errors.append((pos, 'missing .'))

    def checkLineLength(self, line, pos, limit, single_lines):
        linesplit = line.split('\\n')
        if len(linesplit) > 1:
            il = 0
            for f in linesplit:
                il = il + 1
                if len(f) > limit:
                    lenf = self.trueStringLen(f)
                    if lenf > limit:
                        self.errors.append((pos, '%d > %d chars [line %d]'
                                            % (lenf, limit, il)))
        elif single_lines:
            f = linesplit[0]
            if len(f) > limit:
                lenf = self.trueStringLen(f)
                if lenf > limit:
                    self.errors.append((pos, '%d > %d chars'
                                        % (lenf, limit)))

    def trueStringLen(self, f):
        unicode = 0
        for i in f:
            if ord(i) > 127: unicode = unicode + 1
        return len(f) - unicode / 2

    def getMsgstr(self):
        msgstr = ''
        for l, m, i, s in self.data:
            if i != "":
                s = s.replace('\\n', '\n').replace('\\t', '\t')
                s = s.replace('_', '')
                msgstr = msgstr + s + '\n'
        return msgstr

    def getCleanMsgstr(self):
        msgstr = self.getMsgstr()
        msgstr.replace('&', '')
        return msgstr

    def getErrors(self):
        return self.errors

    def hasErrors(self):
        return len(self.errors) > 0
